from typing import Dict

TYPE_MAP: Dict[str, str] = {
    "bool": "boolean",
    "str": "string",
    "int": "number",
    "float": "number",
    "complex": "number",
    "Any": "any",
    "Dict": "Record<any, any>",
    "List": "any[]",
    "Tuple": "[any]",
    "Union": "any",
}

SUBSCRIPT_FORMAT_MAP: Dict[str, str] = {
    "Dict": "Record<{0}>",
    "List": "{0}[]",
    "Optional": "{0} | null",
    "Tuple": "[{0}]",
    "Union": "{0}",
}


def convert_regular_type(pytype):
    if pytype in TYPE_MAP.keys():
        return TYPE_MAP[pytype]
    else:
        return pytype


def convert_subscript_type(pytype, subsripttype):
    if pytype in SUBSCRIPT_FORMAT_MAP.keys():
        return SUBSCRIPT_FORMAT_MAP[pytype].format(subsripttype)
    else:
        return pytype




from dataclasses import dataclass, field
from typing import List

from .Glyph import Glyph


@dataclass
class CharacterMap:
    # Only used for introspection
    numberOfGlyphs: int = 0

    glyphs: List[Glyph] = field(default_factory=list)

    def to_dict(self):
        glyphs = [] if self.glyphs is None else [glyph.__dict__ for glyph in self.glyphs]
        return {
            "numberOfGlyphs": len(glyphs),
            "glyphs": glyphs
        }



from dataclasses import dataclass, field

from .CharacterMap import CharacterMap
from .FontStandard import FontStandard
from .Meta import Meta


@dataclass
class Font:
    name: str = ""
    meta: Meta = field(default_factory=Meta)
    characterMap: CharacterMap = field(default_factory=CharacterMap)
    type: FontStandard = FontStandard.OpenType
    dataUri: str = ""

    def to_dict(self):
        meta = Meta().__dict__ if self.meta is None else self.meta.__dict__
        characterMap = CharacterMap().to_dict() if self.characterMap is None else self.characterMap.to_dict()
        myType = "" if self.type is None else str(self.type)
        return {
            "name": self.name,
            "meta": meta,
            "type": myType,
            "characterMap": characterMap,
            "dataUri": self.dataUri,
        }



import inspect
import json
from enum import Enum

from ..fontmodel.CharacterMap import CharacterMap
from ..fontmodel.Font import Font
from ..fontmodel.FontStandard import FontStandard
from ..fontmodel.Glyph import Glyph
from ..fontmodel.Meta import Meta

from ..typescript.pytsconverter import convert_subscript_type
from ..typescript.pytsconverter import convert_regular_type


def get_typescript_definitions():
    return get_all_definitions()


def is_enum(cls):
    return inspect.isclass(cls) and issubclass(cls, Enum)


def get_class_definition(clazz):
    fontClassDefinition = inspect.signature(clazz.__init__)
    fontDict = {}

    if is_enum(clazz):
        enum_str = f"export enum {clazz.__name__} {{\n"
        for index, member in enumerate(clazz):
            if index < len(clazz) - 1:
                enum_str += f"  {member.name},\n"
            else:
                enum_str += f"  {member.name}\n"
        enum_str += "}"
        return enum_str

    for param_name, param in fontClassDefinition.parameters.items():
        if param_name is not "self":
            param_type = param.annotation
            if hasattr(param_type, "__origin__") and param_type.__origin__ == list:
                list_item_type = param_type.__args__[0].__name__
                tstype = convert_subscript_type(pytype=param.annotation.__name__, subsripttype=str(list_item_type))
                fontDict[param_name] = str(tstype)
            else:
                tstype = convert_regular_type(param.annotation.__name__)
                fontDict[param_name] = str(tstype)
    return f"export interface {clazz.__name__} {json.dumps(fontDict, indent=2).replace("\"", "")}"


def get_all_definitions():
    tsdefinitions = {}
    for cls in [FontStandard, Glyph, CharacterMap, Meta, Font]:
        tsdefinitions[cls.__name__] = get_class_definition(cls)
    return tsdefinitions



from enum import Enum


def toFontStandard(suffix: str):
    if suffix == ".otf":
        return FontStandard.OpenType
    if suffix == ".ttf":
        return FontStandard.TrueType
    return None


class FontStandard(Enum):
    OpenType = 'OpenType'
    TrueType = 'TrueType'



from src.tsfontconverter.fontmodel.CharacterMap import CharacterMap
from src.tsfontconverter.fontmodel.Font import Font
from src.tsfontconverter.fontmodel.FontStandard import FontStandard
from src.tsfontconverter.fontmodel.Glyph import Glyph
from src.tsfontconverter.fontmodel.Meta import Meta
from src.tsfontconverter.typescript.pytsconverter import convert_subscript_type
from src.tsfontconverter.fontparser.reflectionutils import get_class_definition

font_definition = '''export interface Font {
  name: string,
  meta: Meta,
  characterMap: CharacterMap,
  type: FontStandard,
  dataUri: string
}'''
character_map_definition = '''export interface CharacterMap {
  numberOfGlyphs: number,
  glyphs: Glyph[]
}'''
glyph_definition = '''export interface Glyph {
  glyphId: string,
  unicode: string
}'''
meta_definition = '''export interface Meta {
  designerUrl: string,
  licenseUrl: string,
  manufacturerUrl: string,
  description: string,
  sampleText: string
}'''
fontstandard_enum_definition = '''export enum FontStandard {
  OpenType,
  TrueType
}'''


def testPytsconverter():
    a = convert_subscript_type("Dict", "Glyph")
    b = convert_subscript_type("List", "Meta")
    assert a == "Record<Glyph>"
    assert b == "Meta[]"


def test_all():
    test_font_definition()
    test_meta_definition()
    test_glyph_definition()
    test_character_map_definition()
    test_fontstandard_enum()


def test_fontstandard_enum():
    tsfc_fontstandard_enum_definition = get_class_definition(FontStandard)
    print(tsfc_fontstandard_enum_definition)
    assert tsfc_fontstandard_enum_definition == fontstandard_enum_definition


def test_font_definition():
    tsfc_font_definition = get_class_definition(Font)
    print(tsfc_font_definition)
    assert tsfc_font_definition == font_definition


def test_character_map_definition():
    tsfc_character_map_definition = get_class_definition(CharacterMap)
    print(tsfc_character_map_definition)
    assert tsfc_character_map_definition == character_map_definition


def test_glyph_definition():
    tsfc_glyph_map_definition = get_class_definition(Glyph)
    print(tsfc_glyph_map_definition)
    assert tsfc_glyph_map_definition == glyph_definition


def test_meta_definition():
    tsfc_meta_map_definition = get_class_definition(Meta)
    print(tsfc_meta_map_definition)
    assert tsfc_meta_map_definition == meta_definition



from src.tsfontconverter.fontparser.reflectionutils import get_typescript_definitions
from src.tsfontconverter.tsfontconverter import create_font_list
from dotenv import load_dotenv
import os

# Load environment variables from .env file
load_dotenv()

FONTDIRPATH = os.getenv("FONTDIRPATH")
FONT_LIST_PATH = os.getenv("FONT_LIST_PATH")
TYPEDEF_PATH = os.getenv("TYPEDEF_PATH")

FONT_LIST_DATA_START = '''import { Font, FontStandard } from "./font";

const fonts: Font[] = '''


def test_tsfontconverter():
    json_data = create_font_list(fontdirpath=FONTDIRPATH)
    with open(FONT_LIST_PATH, "w") as tsfile:
        tsfile.write(FONT_LIST_DATA_START)
        tsfile.write(json_data)

    typescript_definitions = get_typescript_definitions()
    with open(TYPEDEF_PATH, "w") as tsfile:
        for clazz_name in typescript_definitions:
            typescript_definition = typescript_definitions[clazz_name]
            tsfile.write(typescript_definition)
            tsfile.write("\n\n")



import os
import json
import base64
import re

from fontTools.ttLib import TTFont

from ..fontmodel.CharacterMap import CharacterMap
from ..fontmodel.Font import Font
from ..fontmodel.FontStandard import toFontStandard

from ..fontmodel.Glyph import Glyph
from ..fontmodel.Meta import Meta


def create_font_info(glyphs_unicode, fontname, font_metadata):
    font = Font()
    font.name = fontname
    font.meta = font_metadata
    font.characterMap = CharacterMap()
    characterMap = {}
    for glyph, unicode_code in glyphs_unicode:
        characterMap[glyph] = unicode_code
    for glyphId in characterMap:
        glyph = Glyph()
        glyph.glyphId = glyphId
        glyph.unicode = characterMap[glyphId]
        font.characterMap.glyphs.append(glyph)
    font.characterMap.numberOfGlyphs = len(font.characterMap.glyphs)
    return font


def extract_metadata(font):
    naming_table = font['name']
    font_metadata = Meta()
    for record in naming_table.names:
        if record.nameID == 2:  # Description
            font_metadata.description = record.toUnicode()
        elif record.nameID == 14:  # License URL
            font_metadata.licenseUrl = record.toUnicode()
        elif record.nameID == 12:  # Designer URL
            font_metadata.designerUrl = record.toUnicode()
        elif record.nameID == 11:  # Manufacturer URL
            font_metadata.manufacturerUrl = record.toUnicode()
        elif record.nameID == 10:  # Sample text
            font_metadata.sampleText = record.toUnicode()
    return font_metadata


def extract_glyphs_and_unicode(font):
    glyph_list = []

    # Check if the font is TrueType or CFF
    is_truetype = 'glyf' in font
    is_cff = 'CFF ' in font

    if is_truetype:
        cmap = font['cmap'].getBestCmap()
        for glyph_id, unicode_char in cmap.items():
            glyph_list.append((unicode_char, hex(glyph_id)))

    elif is_cff:
        for table in font['cmap'].tables:
            for code, name in table.cmap.items():
                glyph_list.append((name, hex(code)))
    return list(filter(lambda item: item is not None, glyph_list))


def read_font_file(file_path):
    font = TTFont(file_path)
    # Process the font file here
    return font


def create_base64_encoded_data_uri(fontpath):
    with open(fontpath, "rb") as image_file:
        return f"data:font/opentype;base64,{base64.b64encode(image_file.read()).decode("utf-8")}"


class FontParser:

    def __init__(self, fontdirpath):
        print(f"FontParser initialized {fontdirpath}")
        self._fontdirpath = fontdirpath

    @property
    def fontdirpath(self):
        return self._fontdirpath

    @fontdirpath.setter
    def fontdirpath(self, fontdirpath):
        self._fontdirpath = fontdirpath

    def list_font_files(self):
        myList = os.listdir(self._fontdirpath)
        fontinfolist = []
        for fontname in myList:
            fontpath = self._fontdirpath + "/" + fontname
            if fontpath.endswith(".ttf") or fontpath.endswith(".otf"):
                fontStandard = toFontStandard(os.path.splitext(fontpath)[-1])
                font = TTFont(fontpath)
                glyphs_unicode = extract_glyphs_and_unicode(font)
                fontinfo = create_font_info(glyphs_unicode, os.path.splitext(fontname)[0], extract_metadata(font))
                fontinfo.dataUri = create_base64_encoded_data_uri(fontpath)
                fontinfo.type = fontStandard
                fontinfolist.append(fontinfo)
        font_info_dicts = [font_info.to_dict() for font_info in fontinfolist]
        json_str = json.dumps(font_info_dicts, indent=2)
        json_str = re.sub(r'"(FontStandard\.(OpenType|TrueType))"', r'\1', json_str)
        json_str = re.sub(r'\"(.*?)\":', r'\1:', json_str)
        return f"{json_str}\n\nexport default fonts;"



from .fontparser.fontparser import FontParser

def main():
    """Entry point for the application script"""
    print("TSFontConverter entrypoint main")


def convert_font(font):
    # Add your font conversion logic here
    print("converting font: {font}".format(font=str(font)))


def create_font_list(fontdirpath):
    fontparser = FontParser(fontdirpath=fontdirpath)
    return fontparser.list_font_files()


